@@ -16,6 +16,13 @@ def main():
     print("DEBUG: running demo script in \"{:s}\"".format(sys.argv[0]))
 
     parser = argparse.ArgumentParser()
+    parser.add_argument(
+        "command",
+        nargs='?',
+        default="create",
+        help="the command to run",
+        choices=["create","delete"]
+    )
     parser.add_argument(
         "-n", "--network",
         help="The name of your demo network"
@@ -29,30 +36,54 @@ def main():
         dest="network_group",
         help="The shortname of a Network Group (default is the first, typically singular, Group known to this Org)"
     )
+    parser.add_argument(
+        "-s", "--network-size",
+        dest="size",
+        default="small",
+        help="Billable Network size to create",
+        choices=["small","medium","large"]
+    )
+    parser.add_argument(
+        "-v", "--network-version",
+        dest="version",
+        default="default",
+        help="Network product version: \"default\", \"latest\", or semver"
+    )
     parser.add_argument(
         "-p", "--create-private",
         dest="private",
         default=False,
         action="store_true",
-        help="Also create private Services for the Docker Compose Demo"
+        help="Also create private Endpoint-hosted Services for the optional Docker Compose portion of the quickstart"
     )
     parser.add_argument(
-        "-d", "--create-dialer",
-        dest="dialer",
+        "-c", "--create-client",
+        dest="client",
         default=False,
         action="store_true",
-        help="Also create a Linux dialer for the Docker Compose Demo"
+        help="Also create a client Endpoint for the optional Linux portion of the quickstart"
     )
     parser.add_argument("--credentials",
         default=None,
         help="path to API account credentials JSON file overrides NETFOUNDRY_API_ACCOUNT"
     )
-    parser.add_argument("--regions",
+    parser.add_argument("--provider",
+        default="AWS",
+        required=False,
+        help="cloud provider to host Edge Routers",
+        choices=["AWS", "AZURE", "GCP", "ALICLOUD", "NETFOUNDRY", "OCP"]
+    )
+    regions_group = parser.add_mutually_exclusive_group(required=True)
+    regions_group.add_argument("--regions",
         default=["Americas"],
         nargs="+",
-        required=False,
-        help="space-separated one or more major geographic regions in which to place Edge Routers for overlay fabric: "
-                +' '.join(["Americas", "EuropeMiddleEastAfrica", "AsiaPacific"])
+        help="space-separated one or more major geographic regions in which to place Edge Routers for overlay fabric",
+        choices=["Americas", "EuropeMiddleEastAfrica", "AsiaPacific"]
+    )
+    regions_group.add_argument("--location-codes",
+        dest="location_codes",
+        nargs="+",
+        help="cloud location codes in which to host Edge Routers"
     )
     parser.add_argument("--proxy",
         default=None,
@@ -80,46 +111,79 @@ def main():
     if network_name in network_group.networks_by_name.keys():
         # use the Network
         network = netfoundry.Network(network_group, network_name=network_name)
-        network.wait_for_status("PROVISIONED",wait=999,progress=True)
-    else:
-        network_id = network_group.create_network(name=network_name,size="small")['id'] # use "medium" unless demo
+        if args.command == "create":
+            network.wait_for_status("PROVISIONED",wait=999,progress=True)
+        elif args.command == "delete":
+            if query_yes_no("Permanently destroy Network \"{network_name}\" now?".format(network_name=network_name)):
+                network.delete_network(progress=True)
+            else:
+                print("Not deleting Network \"{network_name}\".".format(network_name=network_name))
+            sys.exit()
+    elif args.command == "create":
+        network_id = network_group.create_network(name=network_name,size=args.size,version=args.version)['id']
         network = netfoundry.Network(network_group, network_id=network_id)
         network.wait_for_status("PROVISIONED",wait=999,progress=True)
 
     # existing hosted ERs
     hosted_edge_routers = network.edge_routers(only_hosted=True)
     # a list of places where Endpoints are dialing from
-    geo_regions = args.regions
+
     # a list of locations to place one hosted ER
     fabric_placements = list()
-    fabric_placement_count = 1
-    for region in geo_regions:
-        data_center_ids = [dc['id'] for dc in network.aws_geo_regions[region]]
-        existing_count = len([er for er in hosted_edge_routers if er['dataCenterId'] in data_center_ids])
-        if existing_count < fabric_placement_count:
-            choice = random.choice(network.aws_geo_regions[region])
-            # append the current major region to the randomly-chosen data center object
-            #   so we can use it as a role attribute when we create the hosted Edge Router
-            choice['geoRegion'] = region
-            fabric_placements += [choice]
-        else:
-            print("INFO: found at least {count} hosted Edge Router(s) in {major}".format(count=fabric_placement_count, major=region))
+    if args.location_codes:
+        for location in args.location_codes:
+            data_centers = [dc for dc in network.get_edge_router_data_centers(provider=args.provider,location_code=location)]
+            data_center = data_centers[0]
+            existing_count = len([er for er in hosted_edge_routers if er['dataCenterId'] == data_center['id']])
+            if existing_count < 1:
+                fabric_placements += [data_center]
+            else:
+                print("INFO: found a hosted Edge Router(s) in {location}".format(location=location))
 
-    for location in fabric_placements:
-        er = network.create_edge_router(
-            name=location['locationName'],
-            attributes=[
-                "#defaultRouters",
-                "#"+location['locationCode'],
-                "#"+location['geoRegion']
-            ],
-            data_center_id=location['id']
-        )
-        hosted_edge_routers += [er]
-        print("INFO: Placed Edge Router in {major} ({location_name})".format(
-            major=location['geoRegion'],
-            location_name=location['locationName']
-        ))
+        for location in fabric_placements:
+            er = network.create_edge_router(
+                name=location['locationName']+" ["+location['provider']+"]",
+                attributes=[
+                    "#defaultRouters",
+                    "#"+location['locationCode'],
+                    "#"+location['provider']
+                ],
+                data_center_id=location['id']
+            )
+            hosted_edge_routers += [er]
+            print("INFO: Placed Edge Router in {provider} ({location_name})".format(
+                provider=location['provider'],
+                location_name=location['locationName']
+            ))
+    elif args.regions:
+        geo_regions = args.regions
+        for region in geo_regions:
+            data_center_ids = [dc['id'] for dc in network.aws_geo_regions[region]]
+            existing_count = len([er for er in hosted_edge_routers if er['dataCenterId'] in data_center_ids])
+            if existing_count < 1:
+                choice = random.choice(network.aws_geo_regions[region])
+                # append the current major region to the randomly-chosen data center object
+                #   so we can use it as a role attribute when we create the hosted Edge Router
+                choice['geoRegion'] = region
+                fabric_placements += [choice]
+            else:
+                print("INFO: found a hosted Edge Router(s) in {major}".format(major=region))
+
+        for location in fabric_placements:
+            er = network.create_edge_router(
+                name=location['locationName']+" ["+location['geoRegion']+"]",
+                attributes=[
+                    "#defaultRouters",
+                    "#"+location['locationCode'],
+                    "#"+location['geoRegion']
+                ],
+                data_center_id=location['id']
+            )
+            hosted_edge_routers += [er]
+            print("INFO: Placed Edge Router in {major} ({location_name})".format(
+                major=location['geoRegion'],
+                location_name=location['locationName']
+            ))
 
     for router_id in [r['id'] for r in hosted_edge_routers]:
         try:
@@ -156,7 +220,7 @@ def main():
         print("INFO: found Endpoint \"{:s}\"".format(client2['name']))
     clients += [client2]
 
-    if args.dialer:
+    if args.client:
         client3_name = "Linux1"
         if not client3_name in [end['name'] for end in endpoints]:
             # create an Endpoint for the dialing device that will access Services
@@ -308,9 +372,9 @@ def main():
     # wait for customer router to be PROVISIONED so that registration will be available 
     try:
         network.wait_for_status("PROVISIONED",id=customer_router['id'],type="edge-router",wait=999,progress=True)
+        customer_router_registration = network.get_edge_router_registration(id=customer_router['id'])
     except:
         raise
-    customer_router_registration = network.get_edge_router_registration(id=customer_router['id'])
     print("INFO: Ready to register branch exit Edge Router {name} with key {key} (expires {expiry})".format(
         name=customer_router_name,
         key=customer_router_registration['registrationKey'],
@@ -329,10 +393,42 @@ def main():
         app_wan1 = [aw for aw in app_wans if aw['name'] == app_wan1_name][0]
         print("INFO: found AppWAN \"{:s}\"".format(app_wan1['name']))
 
-    print("SUCCESS! The next step is to enroll one or more of your dialer Endpoints on some device(s) and visit one of the demo Service URLs described in the demo document ({doc})."
+    print("SUCCESS! The next step is to enroll one or more of your client Endpoints on some device(s) and visit one of the demo Service URLs described in the demo document ({doc})."
             "You may also log in to the web console ({nfconsole}) to play with your Network".format(doc="https://developer.netfoundry.io/v2/tools/#demos",nfconsole=network_group.nfconsole))
     for svc in network.services():
         print("* {name}:\thttp://{url}/".format(name=svc['name'],url=svc['model']['clientIngress']['host']))
 
+def query_yes_no(question: str, default: str="no"):
+    """Ask a yes/no question via input() and return their answer.
+
+    "question" is a string that is presented to the user.
+    "default" is the presumed answer if the user just hits <Enter>.
+        It must be "yes" (the default), "no" or None (meaning
+        an answer is required of the user).
+
+    The "answer" return value is True for "yes" or False for "no".
+    """
+    valid = {"yes": True, "y": True, "ye": True,
+            "no": False, "n": False}
+    if default is None:
+        prompt = " [y/n] "
+    elif default == "yes":
+        prompt = " [Y/n] "
+    elif default == "no":
+        prompt = " [y/N] "
+    else:
+        raise ValueError("invalid default answer: '%s'" % default)
+
+    while True:
+        sys.stdout.write(question + prompt)
+        choice = input().lower()
+        if default is not None and choice == '':
+            return valid[default]
+        elif choice in valid:
+            return valid[choice]
+        else:
+            sys.stdout.write("Please respond with 'yes' or 'no' "
+                            "(or 'y' or 'n').\n")
+
 if __name__ == '__main__':
     main()
\ No newline at end of file
